package cn.com.infosec.jce.provider;

import cn.com.infosec.asn1.ASN1Object;
import cn.com.infosec.asn1.ASN1Sequence;
import cn.com.infosec.asn1.DERObjectIdentifier;
import cn.com.infosec.asn1.cryptopro.CryptoProObjectIdentifiers;
import cn.com.infosec.asn1.oiw.OIWObjectIdentifiers;
import cn.com.infosec.asn1.pkcs.PKCSObjectIdentifiers;
import cn.com.infosec.asn1.pkcs.PrivateKeyInfo;
import cn.com.infosec.asn1.pkcs.RSAPrivateKeyStructure;
import cn.com.infosec.asn1.x509.SubjectPublicKeyInfo;
import cn.com.infosec.asn1.x9.X9ObjectIdentifiers;
import cn.com.infosec.crypto.RuntimeCryptoException;
import cn.com.infosec.jce.interfaces.ElGamalPrivateKey;
import cn.com.infosec.jce.interfaces.ElGamalPublicKey;
import cn.com.infosec.jce.spec.ElGamalPrivateKeySpec;
import cn.com.infosec.jce.spec.ElGamalPublicKeySpec;
import cn.com.infosec.jce.spec.GOST3410PrivateKeySpec;
import cn.com.infosec.jce.spec.GOST3410PublicKeySpec;
import java.math.BigInteger;
import java.security.InvalidKeyException;
import java.security.Key;
import java.security.KeyFactorySpi;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.interfaces.DSAPrivateKey;
import java.security.interfaces.DSAPublicKey;
import java.security.interfaces.ECPrivateKey;
import java.security.interfaces.ECPublicKey;
import java.security.interfaces.RSAPrivateCrtKey;
import java.security.interfaces.RSAPrivateKey;
import java.security.interfaces.RSAPublicKey;
import java.security.spec.DSAPrivateKeySpec;
import java.security.spec.DSAPublicKeySpec;
import java.security.spec.ECPoint;
import java.security.spec.ECPrivateKeySpec;
import java.security.spec.ECPublicKeySpec;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.KeySpec;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.RSAPrivateCrtKeySpec;
import java.security.spec.RSAPrivateKeySpec;
import java.security.spec.RSAPublicKeySpec;
import java.security.spec.X509EncodedKeySpec;
import java.util.Arrays;
import javax.crypto.interfaces.DHPrivateKey;
import javax.crypto.interfaces.DHPublicKey;
import javax.crypto.spec.DHPrivateKeySpec;
import javax.crypto.spec.DHPublicKeySpec;

public abstract class JDKKeyFactory extends KeyFactorySpi {
   public static final byte[] ALG_PARAMETER_SM2_INFOSEC = new byte[]{6, 10, 42, -127, 28, -127, 69, -50, 61, 3, 1, 7};
   public static final byte[] ALG_PARAMETER_SM2_INFOSEC_NEW = new byte[]{6, 8, 42, -127, 28, -49, 85, 1, -126, 45};
   public static final byte[] ALG_PARAMETER_SM2_DETACH = new byte[]{2, 1, 7};
   // $FF: synthetic field
   static Class class$0;
   // $FF: synthetic field
   static Class class$1;
   // $FF: synthetic field
   static Class class$2;
   // $FF: synthetic field
   static Class class$3;
   // $FF: synthetic field
   static Class class$4;
   // $FF: synthetic field
   static Class class$5;
   // $FF: synthetic field
   static Class class$6;
   // $FF: synthetic field
   static Class class$7;
   // $FF: synthetic field
   static Class class$8;
   protected boolean elGamalFactory = false;

   static PublicKey createPublicKeyFromDERStream(byte[] in) {
      return createPublicKeyFromPublicKeyInfo(new SubjectPublicKeyInfo((ASN1Sequence)ASN1Object.fromByteArray(in)));
   }

   static PublicKey createPublicKeyFromPublicKeyInfo(SubjectPublicKeyInfo info) {
      DERObjectIdentifier var1;
      if (RSAUtil.isRsaOid(var1 = info.getAlgorithmId().getObjectId())) {
         return new JCERSAPublicKey(info);
      } else if (var1.equals(PKCSObjectIdentifiers.dhKeyAgreement)) {
         return new JCEDHPublicKey(info);
      } else if (var1.equals(X9ObjectIdentifiers.dhpublicnumber)) {
         return new JCEDHPublicKey(info);
      } else if (var1.equals(OIWObjectIdentifiers.elGamalAlgorithm)) {
         return new JCEElGamalPublicKey(info);
      } else if (var1.equals(X9ObjectIdentifiers.id_dsa)) {
         return new JDKDSAPublicKey(info);
      } else if (var1.equals(OIWObjectIdentifiers.dsaWithSHA1)) {
         return new JDKDSAPublicKey(info);
      } else if (var1.equals(X9ObjectIdentifiers.id_ecPublicKey)) {
         byte[] var10000;
         boolean var10001;
         try {
            var10000 = info.getAlgorithmId().getParameters().getDERObject().getDEREncoded();
         } catch (Exception var7) {
            var10001 = false;
            throw new RuntimeCryptoException("Decode ecc public failed");
         }

         byte[] var8 = var10000;

         boolean var9;
         try {
            var9 = Arrays.equals(var10000, ALG_PARAMETER_SM2_INFOSEC_NEW);
         } catch (Exception var6) {
            var10001 = false;
            throw new RuntimeCryptoException("Decode ecc public failed");
         }

         if (!var9) {
            try {
               var9 = Arrays.equals(var8, ALG_PARAMETER_SM2_INFOSEC);
            } catch (Exception var5) {
               var10001 = false;
               throw new RuntimeCryptoException("Decode ecc public failed");
            }

            if (!var9) {
               try {
                  var9 = Arrays.equals(ALG_PARAMETER_SM2_DETACH, var8);
               } catch (Exception var4) {
                  var10001 = false;
                  throw new RuntimeCryptoException("Decode ecc public failed");
               }

               if (!var9) {
                  try {
                     return new JCEECPublicKey(info);
                  } catch (Exception var2) {
                     var10001 = false;
                     throw new RuntimeCryptoException("Decode ecc public failed");
                  }
               }
            }
         }

         try {
            return new JCESM2PublicKey(info.getEncoded());
         } catch (Exception var3) {
            var10001 = false;
            throw new RuntimeCryptoException("Decode ecc public failed");
         }
      } else if (var1.equals(CryptoProObjectIdentifiers.gostR3410_94)) {
         return new JDKGOST3410PublicKey(info);
      } else if (var1.equals(CryptoProObjectIdentifiers.gostR3410_2001)) {
         return new JCEECPublicKey(info);
      } else {
         throw new RuntimeException("algorithm identifier " + var1 + " in key not recognised");
      }
   }

   static PrivateKey createPrivateKeyFromDERStream(byte[] in) {
      return createPrivateKeyFromPrivateKeyInfo(new PrivateKeyInfo((ASN1Sequence)ASN1Object.fromByteArray(in)));
   }

   static PrivateKey createPrivateKeyFromPrivateKeyInfo(PrivateKeyInfo info) {
      DERObjectIdentifier var1;
      if (RSAUtil.isRsaOid(var1 = info.getAlgorithmId().getObjectId())) {
         return new JCERSAPrivateCrtKey(info);
      } else if (var1.equals(PKCSObjectIdentifiers.dhKeyAgreement)) {
         return new JCEDHPrivateKey(info);
      } else if (var1.equals(OIWObjectIdentifiers.elGamalAlgorithm)) {
         return new JCEElGamalPrivateKey(info);
      } else if (var1.equals(X9ObjectIdentifiers.id_dsa)) {
         return new JDKDSAPrivateKey(info);
      } else if (var1.equals(X9ObjectIdentifiers.id_ecPublicKey)) {
         Exception var10000;
         label82: {
            byte[] var9;
            boolean var10001;
            try {
               var9 = info.getAlgorithmId().getParameters().getDERObject().getDEREncoded();
            } catch (Exception var7) {
               var10000 = var7;
               var10001 = false;
               break label82;
            }

            byte[] var8 = var9;

            boolean var10;
            try {
               var10 = Arrays.equals(var9, ALG_PARAMETER_SM2_INFOSEC_NEW);
            } catch (Exception var6) {
               var10000 = var6;
               var10001 = false;
               break label82;
            }

            if (!var10) {
               try {
                  var10 = Arrays.equals(var8, ALG_PARAMETER_SM2_INFOSEC);
               } catch (Exception var5) {
                  var10000 = var5;
                  var10001 = false;
                  break label82;
               }

               if (!var10) {
                  try {
                     var10 = Arrays.equals(ALG_PARAMETER_SM2_DETACH, var8);
                  } catch (Exception var4) {
                     var10000 = var4;
                     var10001 = false;
                     break label82;
                  }

                  if (!var10) {
                     try {
                        return new JCEECPrivateKey(info);
                     } catch (Exception var2) {
                        var10000 = var2;
                        var10001 = false;
                        break label82;
                     }
                  }
               }
            }

            try {
               return new JCESM2PrivateKey(info.getEncoded());
            } catch (Exception var3) {
               var10000 = var3;
               var10001 = false;
            }
         }

         var10000.printStackTrace();
         throw new RuntimeCryptoException("Decode ecc public failed");
      } else if (var1.equals(CryptoProObjectIdentifiers.gostR3410_94)) {
         return new JDKGOST3410PrivateKey(info);
      } else if (var1.equals(CryptoProObjectIdentifiers.gostR3410_2001)) {
         return new JCEECPrivateKey(info);
      } else {
         throw new RuntimeException("algorithm identifier " + var1 + " in key not recognised");
      }
   }

   protected PrivateKey engineGeneratePrivate(KeySpec keySpec) {
      if (keySpec instanceof PKCS8EncodedKeySpec) {
         try {
            return createPrivateKeyFromDERStream(((PKCS8EncodedKeySpec)keySpec).getEncoded());
         } catch (Exception var2) {
            throw new InvalidKeySpecException(var2.toString());
         }
      } else {
         throw new InvalidKeySpecException("Unknown KeySpec type: " + keySpec.getClass().getName());
      }
   }

   protected PublicKey engineGeneratePublic(KeySpec keySpec) {
      if (keySpec instanceof X509EncodedKeySpec) {
         try {
            return createPublicKeyFromDERStream(((X509EncodedKeySpec)keySpec).getEncoded());
         } catch (Exception var2) {
            throw new InvalidKeySpecException(var2.toString());
         }
      } else {
         throw new InvalidKeySpecException("Unknown KeySpec type: " + keySpec.getClass().getName());
      }
   }

   protected KeySpec engineGetKeySpec(Key key, Class spec) {
      Class var10000;
      Class var17;
      if ((var17 = class$0) == null) {
         try {
            var10000 = Class.forName("java.security.spec.PKCS8EncodedKeySpec");
         } catch (ClassNotFoundException var16) {
            throw new NoClassDefFoundError(var16.getMessage());
         }

         var17 = var10000;
         class$0 = var10000;
      }

      if (spec.isAssignableFrom(var17) && key.getFormat().equals("PKCS#8")) {
         return new PKCS8EncodedKeySpec(key.getEncoded());
      } else {
         if ((var17 = class$1) == null) {
            try {
               var10000 = Class.forName("java.security.spec.X509EncodedKeySpec");
            } catch (ClassNotFoundException var15) {
               throw new NoClassDefFoundError(var15.getMessage());
            }

            var17 = var10000;
            class$1 = var10000;
         }

         if (spec.isAssignableFrom(var17) && key.getFormat().equals("X.509")) {
            return new X509EncodedKeySpec(key.getEncoded());
         } else {
            if ((var17 = class$2) == null) {
               try {
                  var10000 = Class.forName("java.security.spec.RSAPublicKeySpec");
               } catch (ClassNotFoundException var14) {
                  throw new NoClassDefFoundError(var14.getMessage());
               }

               var17 = var10000;
               class$2 = var10000;
            }

            BigInteger var21;
            if (spec.isAssignableFrom(var17) && key instanceof RSAPublicKey) {
               RSAPublicKey var28 = (RSAPublicKey)key;
               RSAPublicKey var35 = var28;
               var21 = var28.getModulus();
               return new RSAPublicKeySpec(var21, var35.getPublicExponent());
            } else {
               if ((var17 = class$3) == null) {
                  try {
                     var10000 = Class.forName("java.security.spec.RSAPrivateKeySpec");
                  } catch (ClassNotFoundException var13) {
                     throw new NoClassDefFoundError(var13.getMessage());
                  }

                  var17 = var10000;
                  class$3 = var10000;
               }

               if (spec.isAssignableFrom(var17) && key instanceof RSAPrivateKey) {
                  RSAPrivateKey var27 = (RSAPrivateKey)key;
                  RSAPrivateKey var33 = var27;
                  var21 = var27.getModulus();
                  return new RSAPrivateKeySpec(var21, var33.getPrivateExponent());
               } else {
                  if ((var17 = class$4) == null) {
                     try {
                        var10000 = Class.forName("java.security.spec.RSAPrivateCrtKeySpec");
                     } catch (ClassNotFoundException var12) {
                        throw new NoClassDefFoundError(var12.getMessage());
                     }

                     var17 = var10000;
                     class$4 = var10000;
                  }

                  if (spec.isAssignableFrom(var17) && key instanceof RSAPrivateCrtKey) {
                     RSAPrivateCrtKey var26 = (RSAPrivateCrtKey)key;
                     RSAPrivateCrtKey var31 = var26;
                     RSAPrivateCrtKey var34 = var26;
                     RSAPrivateCrtKey var10004 = var26;
                     RSAPrivateCrtKey var10005 = var26;
                     RSAPrivateCrtKey var10006 = var26;
                     RSAPrivateCrtKey var10007 = var26;
                     RSAPrivateCrtKey var10008 = var26;
                     var21 = var26.getModulus();
                     BigInteger key1 = var10008.getPublicExponent();
                     BigInteger spec1 = var10007.getPrivateExponent();
                     BigInteger var3 = var10006.getPrimeP();
                     BigInteger var4 = var10005.getPrimeQ();
                     BigInteger var5 = var10004.getPrimeExponentP();
                     BigInteger var6 = var34.getPrimeExponentQ();
                     BigInteger var7 = var31.getCrtCoefficient();
                     return new RSAPrivateCrtKeySpec(var21, key1, spec1, var3, var4, var5, var6, var7);
                  } else {
                     if ((var17 = class$5) == null) {
                        try {
                           var10000 = Class.forName("javax.crypto.spec.DHPrivateKeySpec");
                        } catch (ClassNotFoundException var11) {
                           throw new NoClassDefFoundError(var11.getMessage());
                        }

                        var17 = var10000;
                        class$5 = var10000;
                     }

                     BigInteger var30;
                     if (spec.isAssignableFrom(var17) && key instanceof DHPrivateKey) {
                        DHPrivateKey var25 = (DHPrivateKey)key;
                        var30 = var25.getX();
                        DHPrivateKey var32 = var25;
                        var21 = var25.getParams().getP();
                        return new DHPrivateKeySpec(var30, var21, var32.getParams().getG());
                     } else {
                        if ((var17 = class$6) == null) {
                           try {
                              var10000 = Class.forName("javax.crypto.spec.DHPublicKeySpec");
                           } catch (ClassNotFoundException var10) {
                              throw new NoClassDefFoundError(var10.getMessage());
                           }

                           var17 = var10000;
                           class$6 = var10000;
                        }

                        if (spec.isAssignableFrom(var17) && key instanceof DHPublicKey) {
                           DHPublicKey var24 = (DHPublicKey)key;
                           var30 = var24.getY();
                           DHPublicKey var10003 = var24;
                           var21 = var24.getParams().getP();
                           return new DHPublicKeySpec(var30, var21, var10003.getParams().getG());
                        } else {
                           if ((var17 = class$7) == null) {
                              try {
                                 var10000 = Class.forName("java.security.spec.ECPublicKeySpec");
                              } catch (ClassNotFoundException var9) {
                                 throw new NoClassDefFoundError(var9.getMessage());
                              }

                              var17 = var10000;
                              class$7 = var10000;
                           }

                           if (spec.isAssignableFrom(var17) && key instanceof ECPublicKey) {
                              ECPublicKey var22 = (ECPublicKey)key;
                              ECPublicKey var29 = var22;
                              ECPoint var23 = var22.getW();
                              return new ECPublicKeySpec(var23, var29.getParams());
                           } else {
                              if ((var17 = class$8) == null) {
                                 try {
                                    var10000 = Class.forName("java.security.spec.ECPrivateKeySpec");
                                 } catch (ClassNotFoundException var8) {
                                    throw new NoClassDefFoundError(var8.getMessage());
                                 }

                                 var17 = var10000;
                                 class$8 = var10000;
                              }

                              if (spec.isAssignableFrom(var17) && key instanceof ECPrivateKey) {
                                 ECPrivateKey var20 = (ECPrivateKey)key;
                                 ECPrivateKey var10002 = var20;
                                 var21 = var20.getS();
                                 return new ECPrivateKeySpec(var21, var10002.getParams());
                              } else {
                                 throw new RuntimeException("not implemented yet " + key + " " + spec);
                              }
                           }
                        }
                     }
                  }
               }
            }
         }
      }
   }

   protected Key engineTranslateKey(Key key) {
      if (key instanceof RSAPublicKey) {
         return new JCERSAPublicKey((RSAPublicKey)key);
      } else if (key instanceof RSAPrivateCrtKey) {
         return new JCERSAPrivateCrtKey((RSAPrivateCrtKey)key);
      } else if (key instanceof RSAPrivateKey) {
         return new JCERSAPrivateKey((RSAPrivateKey)key);
      } else if (key instanceof DHPublicKey) {
         return (Key)(this.elGamalFactory ? new JCEElGamalPublicKey((DHPublicKey)key) : new JCEDHPublicKey((DHPublicKey)key));
      } else if (key instanceof DHPrivateKey) {
         return (Key)(this.elGamalFactory ? new JCEElGamalPrivateKey((DHPrivateKey)key) : new JCEDHPrivateKey((DHPrivateKey)key));
      } else if (key instanceof DSAPublicKey) {
         return new JDKDSAPublicKey((DSAPublicKey)key);
      } else if (key instanceof DSAPrivateKey) {
         return new JDKDSAPrivateKey((DSAPrivateKey)key);
      } else if (key instanceof ElGamalPublicKey) {
         return new JCEElGamalPublicKey((ElGamalPublicKey)key);
      } else if (key instanceof ElGamalPrivateKey) {
         return new JCEElGamalPrivateKey((ElGamalPrivateKey)key);
      } else if (key instanceof ECPublicKey) {
         return new JCEECPublicKey((ECPublicKey)key);
      } else if (key instanceof ECPrivateKey) {
         return new JCEECPrivateKey((ECPrivateKey)key);
      } else {
         throw new InvalidKeyException("key type unknown");
      }
   }

   public static class DH extends JDKKeyFactory {
      protected PrivateKey engineGeneratePrivate(KeySpec keySpec) {
         return (PrivateKey)(keySpec instanceof DHPrivateKeySpec ? new JCEDHPrivateKey((DHPrivateKeySpec)keySpec) : super.engineGeneratePrivate(keySpec));
      }

      protected PublicKey engineGeneratePublic(KeySpec keySpec) {
         return (PublicKey)(keySpec instanceof DHPublicKeySpec ? new JCEDHPublicKey((DHPublicKeySpec)keySpec) : super.engineGeneratePublic(keySpec));
      }
   }

   public static class DSA extends JDKKeyFactory {
      protected PrivateKey engineGeneratePrivate(KeySpec keySpec) {
         return (PrivateKey)(keySpec instanceof DSAPrivateKeySpec ? new JDKDSAPrivateKey((DSAPrivateKeySpec)keySpec) : super.engineGeneratePrivate(keySpec));
      }

      protected PublicKey engineGeneratePublic(KeySpec keySpec) {
         return (PublicKey)(keySpec instanceof DSAPublicKeySpec ? new JDKDSAPublicKey((DSAPublicKeySpec)keySpec) : super.engineGeneratePublic(keySpec));
      }
   }

   public static class EC extends JDKKeyFactory {
      String algorithm;

      public EC() {
         this("EC");
      }

      public EC(String algorithm) {
         this.algorithm = algorithm;
      }

      protected PrivateKey engineGeneratePrivate(KeySpec keySpec) {
         if (keySpec instanceof PKCS8EncodedKeySpec) {
            Exception var10000;
            label35: {
               boolean var10001;
               JCEECPrivateKey var7;
               try {
                  var7 = (JCEECPrivateKey)JDKKeyFactory.createPrivateKeyFromDERStream(((PKCS8EncodedKeySpec)keySpec).getEncoded());
               } catch (Exception var3) {
                  var10000 = var3;
                  var10001 = false;
                  break label35;
               }

               JCEECPrivateKey keySpec1 = var7;

               try {
                  return new JCEECPrivateKey(this.algorithm, keySpec1);
               } catch (Exception var2) {
                  var10000 = var2;
                  var10001 = false;
               }
            }

            Exception var5 = var10000;
            throw new InvalidKeySpecException(var5.toString());
         } else {
            String var4;
            if (keySpec instanceof cn.com.infosec.jce.spec.ECPrivateKeySpec) {
               var4 = this.algorithm;
               return new JCEECPrivateKey(var4, (cn.com.infosec.jce.spec.ECPrivateKeySpec)keySpec);
            } else if (keySpec instanceof ECPrivateKeySpec) {
               var4 = this.algorithm;
               return new JCEECPrivateKey(var4, (ECPrivateKeySpec)keySpec);
            } else {
               throw new InvalidKeySpecException("Unknown KeySpec type: " + keySpec.getClass().getName());
            }
         }
      }

      protected PublicKey engineGeneratePublic(KeySpec keySpec) {
         if (keySpec instanceof X509EncodedKeySpec) {
            Exception var10000;
            label53: {
               PublicKey var10;
               boolean var10001;
               try {
                  var10 = JDKKeyFactory.createPublicKeyFromDERStream(((X509EncodedKeySpec)keySpec).getEncoded());
               } catch (Exception var5) {
                  var10000 = var5;
                  var10001 = false;
                  break label53;
               }

               PublicKey keySpec1 = var10;

               boolean var11;
               try {
                  var11 = var10 instanceof JCEECPublicKey;
               } catch (Exception var4) {
                  var10000 = var4;
                  var10001 = false;
                  break label53;
               }

               if (!var11) {
                  return keySpec1;
               }

               JCEECPublicKey var12;
               try {
                  var12 = (JCEECPublicKey)keySpec1;
               } catch (Exception var3) {
                  var10000 = var3;
                  var10001 = false;
                  break label53;
               }

               JCEECPublicKey keySpec2 = var12;

               try {
                  return new JCEECPublicKey(this.algorithm, keySpec2);
               } catch (Exception var2) {
                  var10000 = var2;
                  var10001 = false;
               }
            }

            Exception var7 = var10000;
            throw new InvalidKeySpecException(var7.toString());
         } else {
            String var6;
            if (keySpec instanceof cn.com.infosec.jce.spec.ECPublicKeySpec) {
               var6 = this.algorithm;
               return new JCEECPublicKey(var6, (cn.com.infosec.jce.spec.ECPublicKeySpec)keySpec);
            } else if (keySpec instanceof ECPublicKeySpec) {
               var6 = this.algorithm;
               return new JCEECPublicKey(var6, (ECPublicKeySpec)keySpec);
            } else {
               throw new InvalidKeySpecException("Unknown KeySpec type: " + keySpec.getClass().getName());
            }
         }
      }
   }

   public static class ECDH extends EC {
      public ECDH() {
         super("ECDH");
      }
   }

   public static class ECDHC extends EC {
      public ECDHC() {
         super("ECDHC");
      }
   }

   public static class ECDSA extends EC {
      public ECDSA() {
         super("ECDSA");
      }
   }

   public static class ECGOST3410 extends EC {
      public ECGOST3410() {
         super("ECGOST3410");
      }
   }

   public static class ElGamal extends JDKKeyFactory {
      public ElGamal() {
         super.elGamalFactory = true;
      }

      protected PrivateKey engineGeneratePrivate(KeySpec keySpec) {
         if (keySpec instanceof ElGamalPrivateKeySpec) {
            return new JCEElGamalPrivateKey((ElGamalPrivateKeySpec)keySpec);
         } else {
            return (PrivateKey)(keySpec instanceof DHPrivateKeySpec ? new JCEElGamalPrivateKey((DHPrivateKeySpec)keySpec) : super.engineGeneratePrivate(keySpec));
         }
      }

      protected PublicKey engineGeneratePublic(KeySpec keySpec) {
         if (keySpec instanceof ElGamalPublicKeySpec) {
            return new JCEElGamalPublicKey((ElGamalPublicKeySpec)keySpec);
         } else {
            return (PublicKey)(keySpec instanceof DHPublicKeySpec ? new JCEElGamalPublicKey((DHPublicKeySpec)keySpec) : super.engineGeneratePublic(keySpec));
         }
      }
   }

   public static class GOST3410 extends JDKKeyFactory {
      protected PrivateKey engineGeneratePrivate(KeySpec keySpec) {
         return (PrivateKey)(keySpec instanceof GOST3410PrivateKeySpec ? new JDKGOST3410PrivateKey((GOST3410PrivateKeySpec)keySpec) : super.engineGeneratePrivate(keySpec));
      }

      protected PublicKey engineGeneratePublic(KeySpec keySpec) {
         return (PublicKey)(keySpec instanceof GOST3410PublicKeySpec ? new JDKGOST3410PublicKey((GOST3410PublicKeySpec)keySpec) : super.engineGeneratePublic(keySpec));
      }
   }

   public static class RSA extends JDKKeyFactory {
      protected PrivateKey engineGeneratePrivate(KeySpec keySpec) {
         if (keySpec instanceof PKCS8EncodedKeySpec) {
            try {
               return JDKKeyFactory.createPrivateKeyFromDERStream(((PKCS8EncodedKeySpec)keySpec).getEncoded());
            } catch (Exception var3) {
               try {
                  return new JCERSAPrivateCrtKey(new RSAPrivateKeyStructure((ASN1Sequence)ASN1Object.fromByteArray(((PKCS8EncodedKeySpec)keySpec).getEncoded())));
               } catch (Exception var2) {
                  throw new InvalidKeySpecException(var2.toString());
               }
            }
         } else if (keySpec instanceof RSAPrivateCrtKeySpec) {
            return new JCERSAPrivateCrtKey((RSAPrivateCrtKeySpec)keySpec);
         } else if (keySpec instanceof RSAPrivateKeySpec) {
            return new JCERSAPrivateKey((RSAPrivateKeySpec)keySpec);
         } else {
            throw new InvalidKeySpecException("Unknown KeySpec type: " + keySpec.getClass().getName());
         }
      }

      protected PublicKey engineGeneratePublic(KeySpec keySpec) {
         return (PublicKey)(keySpec instanceof RSAPublicKeySpec ? new JCERSAPublicKey((RSAPublicKeySpec)keySpec) : super.engineGeneratePublic(keySpec));
      }
   }

   public static class X509 extends JDKKeyFactory {
   }
}
